@@ -6,8 +6,7 @@
 ;; URL: http://github.com/s-kostyaev/ellama
 ;; Keywords: help local tools
 ;; Package-Requires: ((emacs "28.1") (llm "0.24.0") (plz "0.8") (transient "0.7") (compat "29.1"))
-;; Package-Version: 1.8.7
-;; Package-Revision: 4fa24686efab
+;; Version: 1.9.1
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;; Created: 8th Oct 2023
 
@@ -226,6 +225,15 @@ PROMPT is a prompt string."
 	  (const :tag "By generating name with reasoning LLM based on prompt." ellama-generate-name-by-reasoning-llm)
           (function :tag "By custom function")))
 
+(defcustom ellama-response-process-method 'streaming
+  "How to process LLM response.
+If the default streaming method is too resource-heavy, you can try other
+options."
+  :type `(choice
+	  (const :tag "Streaming" streaming)
+	  (const :tag "Async" async)
+	  (integer :tag "Skip every N messages before process one")))
+
 (defcustom ellama-define-word-prompt-template "Define %s"
   "Prompt template for `ellama-define-word'."
   :type 'string)
@@ -1438,39 +1446,114 @@ failure (with BUFFER current).
 	  (require 'spinner)
 	  (spinner-start ellama-spinner-type))
 	(let* ((handler (ellama--handle-partial insert-text insert-reasoning reasoning-buffer))
-	       (request (llm-chat-streaming
-			 provider
-			 llm-prompt
-			 handler
-			 (lambda (response)
-			   (let ((text (plist-get response :text))
-				 (reasoning (plist-get response :reasoning)))
-			     (funcall handler response)
-			     (when (or ellama--current-session
-				       (not reasoning))
-			       (kill-buffer reasoning-buffer))
-			     (with-current-buffer buffer
-			       (accept-change-group ellama--change-group)
-			       (when ellama-spinner-enabled
-				 (spinner-stop))
-			       (if (and (listp donecb)
-					(functionp (car donecb)))
-				   (mapc (lambda (fn) (funcall fn text))
-					 donecb)
-				 (funcall donecb text))
-			       (when ellama-session-hide-org-quotes
-				 (ellama-collapse-org-quotes))
-			       (setq ellama--current-request nil)
-			       (ellama-request-mode -1))))
-			 (lambda (_ msg)
-			   (with-current-buffer buffer
-			     (cancel-change-group ellama--change-group)
-			     (when ellama-spinner-enabled
-			       (spinner-stop))
-			     (funcall errcb msg)
-			     (setq ellama--current-request nil)
-			     (ellama-request-mode -1)))
-			 t)))
+	       (request (pcase ellama-response-process-method
+			  ('async (llm-chat-async
+				   provider
+				   llm-prompt
+				   (lambda (response)
+				     (let ((text (plist-get response :text))
+					   (reasoning (plist-get response :reasoning)))
+				       (funcall handler response)
+				       (when (or ellama--current-session
+						 (not reasoning))
+					 (kill-buffer reasoning-buffer))
+				       (with-current-buffer buffer
+					 (accept-change-group ellama--change-group)
+					 (when ellama-spinner-enabled
+					   (spinner-stop))
+					 (if (and (listp donecb)
+						  (functionp (car donecb)))
+					     (mapc (lambda (fn) (funcall fn text))
+						   donecb)
+					   (funcall donecb text))
+					 (when ellama-session-hide-org-quotes
+					   (ellama-collapse-org-quotes))
+					 (setq ellama--current-request nil)
+					 (ellama-request-mode -1))))
+				   (lambda (_ msg)
+				     (with-current-buffer buffer
+				       (cancel-change-group ellama--change-group)
+				       (when ellama-spinner-enabled
+					 (spinner-stop))
+				       (funcall errcb msg)
+				       (setq ellama--current-request nil)
+				       (ellama-request-mode -1)))
+				   t))
+			  ('streaming (llm-chat-streaming
+				       provider
+				       llm-prompt
+				       handler
+				       (lambda (response)
+					 (let ((text (plist-get response :text))
+					       (reasoning (plist-get response :reasoning)))
+					   (funcall handler response)
+					   (when (or ellama--current-session
+						     (not reasoning))
+					     (kill-buffer reasoning-buffer))
+					   (with-current-buffer buffer
+					     (accept-change-group ellama--change-group)
+					     (when ellama-spinner-enabled
+					       (spinner-stop))
+					     (if (and (listp donecb)
+						      (functionp (car donecb)))
+						 (mapc (lambda (fn) (funcall fn text))
+						       donecb)
+					       (funcall donecb text))
+					     (when ellama-session-hide-org-quotes
+					       (ellama-collapse-org-quotes))
+					     (setq ellama--current-request nil)
+					     (ellama-request-mode -1))))
+				       (lambda (_ msg)
+					 (with-current-buffer buffer
+					   (cancel-change-group ellama--change-group)
+					   (when ellama-spinner-enabled
+					     (spinner-stop))
+					   (funcall errcb msg)
+					   (setq ellama--current-request nil)
+					   (ellama-request-mode -1)))
+				       t))
+			  ((pred integerp)
+			   (let* ((cnt 0)
+				  (skip-handler
+				   (lambda (request)
+				     (if (= cnt ellama-response-process-method)
+					 (progn
+					   (funcall handler request)
+					   (setq cnt 0))
+				       (cl-incf cnt)))))
+			     (llm-chat-streaming
+			      provider
+			      llm-prompt
+			      skip-handler
+			      (lambda (response)
+				(let ((text (plist-get response :text))
+				      (reasoning (plist-get response :reasoning)))
+				  (funcall handler response)
+				  (when (or ellama--current-session
+					    (not reasoning))
+				    (kill-buffer reasoning-buffer))
+				  (with-current-buffer buffer
+				    (accept-change-group ellama--change-group)
+				    (when ellama-spinner-enabled
+				      (spinner-stop))
+				    (if (and (listp donecb)
+					     (functionp (car donecb)))
+					(mapc (lambda (fn) (funcall fn text))
+					      donecb)
+				      (funcall donecb text))
+				    (when ellama-session-hide-org-quotes
+				      (ellama-collapse-org-quotes))
+				    (setq ellama--current-request nil)
+				    (ellama-request-mode -1))))
+			      (lambda (_ msg)
+				(with-current-buffer buffer
+				  (cancel-change-group ellama--change-group)
+				  (when ellama-spinner-enabled
+				    (spinner-stop))
+				  (funcall errcb msg)
+				  (setq ellama--current-request nil)
+				  (ellama-request-mode -1)))
+			      t))))))
 	  (with-current-buffer buffer
 	    (setq ellama--current-request request)))))))
 
